// apachemod/Globals.cc
// This file is part of Anyterm; see http://anyterm.org/
// (C) 2005 Philip Endecott


// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#include "Globals.hh"

#include <string>

#include "Exception.hh"

#include <boost/lexical_cast.hpp>


using namespace std;


Globals* Globals::g;


Session& Globals::get_session(SessionId id) 
{
  sessions_t::iterator i = sessions.find(id);
  if (i==sessions.end()) {
    throw StrException("No such session '"+boost::lexical_cast<string>(id)+"'");
  }
  return i->second;
}


Session& Globals::create_session(string command, int rows, int cols, int time_out)
{
  expunge_sessions();

  Session new_ses(rows,cols,time_out,command);
  new_ses.touch();
  sessions.insert(make_pair(new_ses.id,new_ses));
  Session& ses(sessions.find(new_ses.id)->second);
  ses.open_req=true;  
  notify_backend();

  for (int c=0; c<2000; c++) {
    ses.report_any_backend_error();
    if (ses.is_open) {
      break;
    }
    usleep(1000);
  }

  if (!ses.is_open) {
    throw StrException("Session failed to open");
  }

  return ses;
}


void Globals::expunge_sessions(void)
{
  for (sessions_t::iterator i = sessions.begin();
       i != sessions.end(); ++i) {
    if (!(i->second.is_open) && !(i->second.open_req)) {
      sessions.erase(i);
      expunge_sessions();
      return;
    }
  }
}


void Globals::notify_backend(void)
{
  Mutex::scoped_lock l(notification_mutex);
  notification=true;
  notification_condition.notify_all();
}
