// apachemod/Globals.hh
// This file is part of Anyterm; see http://anyterm.org/
// (C) 2005 Philip Endecott


// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


// This file defines a class that represents the top-level state of
// the Anyterm module; it is stored in the shared memory and includes
// a list of Anyterm sessions.

#ifndef globals_h
#define globals_h

#include "Session.hh"


class Globals {
private:
  static Globals* g;

public:
  static void set_singleton(Globals* g_) { g=g_; }
  static Globals& singleton() { return *g; }

  Globals(void): notification(false) {};

  Session& get_session(SessionId id);
  Session& create_session(std::string command, int rows, int cols, int time_out);
  void expunge_sessions(void);
  void notify_backend(void);

  // sessions needs a lock
  typedef shared_containers::map<SessionId,Session> sessions_t;
  sessions_t sessions;

  bool notification;
  Mutex notification_mutex;
  Condition notification_condition;
};



#endif

