// apachemod/Session.hh
// This file is part of Anyterm; see http://anyterm.org/
// (C) 2005-2006 Philip Endecott

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifndef Session_hh
#define Session_hh

#include <queue>

extern "C" {
#include <rote/rote.h>
};

#include "Screen.hh"
#include "SessionId.hh"


#ifdef USE_SHMEM

#include "SharedObject.hh"

#include <boost/shmem/sync/shared_mutex.hpp>
#include <boost/shmem/sync/shared_condition.hpp>

namespace shared_containers = SharedObject;

typedef boost::shmem::shared_mutex Mutex;
typedef boost::shmem::shared_condition Condition;

#else

#include <vector>
#include <string>
#include <boost/thread/mutex.hpp>
#include <boost/thread/condition.hpp>

namespace shared_containers = std;

typedef boost::mutex Mutex;
typedef boost::condition Condition;

#endif


class Session {
public:

  const SessionId id;
  const int rows;
  const int cols;
  const int time_out;
  const shared_containers::string command;

  volatile bool open_req;
  volatile bool is_open;
  /*volatile*/ std::queue<char, shared_containers::deque<char> > key_fifo;

  typedef /*volatile*/ Screen<shared_containers::vector<RoteCell> > screen_t;
  screen_t screen;
  volatile bool dirty;
  Condition dirty_condition;
  /*volatile*/ shared_containers::string error_msg;
  volatile bool error;
  shared_containers::string old_html_screen;
  volatile time_t last_access;
  class Backend* be;

  Session(int r, int c, int t, std::string cmd);
  Session(const Session& other);
  ~Session() {};

  Mutex screen_lock;
  void touch(void);
  void report_any_backend_error(void);
  std::string send(std::string k);
  std::string rcv(void);
  void close(void);
};


#endif
