// apachemod/wapterm.cc
// This file is part of Anyterm; see http://anyterm.org/
// (C) 2005 Philip Endecott


// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#include "wapterm.h"

#include "config.hh"
#include "anyterm.h"
#include "html.hh"
#include "Exception.hh"
#include "Session.hh"
#include "Globals.hh"

#include <sstream>

using namespace std;


#define WML_MIME_TYPE "text/vnd.wap.wml"
//#define WML_MIME_TYPE "text/plain"

#define WML_DOCTYPE "<!DOCTYPE wml PUBLIC \"-//WAPFORUM//DTD WML 1.1//EN\" \"http://www.wapforum.org/DTD/wml_1.1.xml\">"

static response_t wml_resp(string s)
{
  return make_pair(WML_MIME_TYPE,s);
}


static string decode_wapkeys(string k)
{
  if (k=="") {
    return "\012";
  }

  bool add_ret = false;
  string d;
  string::size_type p;
  for (p=0; p<k.length()-1; ++p) {
    if (k[p]=='^') {
      ++p;
      if (k[p]=='^') {
	d += '^';
	add_ret = true;
      } else if (k[p]>='a' && k[p]<='z') {
	d += (k[p]-'a'+1);
      } else if (k[p]>='A' && k[p]<='Z') {
	d += (k[p]-'A'+1);
      }
    } else {
      d += k[p];
      if (k[p]!=' ') {
	add_ret = true;
      }
    }
  }
  if (p==k.length()-1) {
    if (k[p]=='^') {
      add_ret = false;
    } else {
      d += k[p];
    }
  }
  if (add_ret) {
    d += "\012";
  }
  return d;
}


static response_t wapterm_i(const CgiParams& params, string command)
{
  Session* sesptr;

  if (params.find("s")==params.end()) {
    sesptr = &Globals::singleton().create_session(command, 8, 18, WAPTERM_TIMEOUT);
  } else {
    SessionId id(params.get("s"));
    sesptr = &Globals::singleton().get_session(id);
  }

  Session& ses(*sesptr);

  ses.report_any_backend_error();

  if (!ses.is_open) {
    throw StrException("not open");
  }

  ses.touch();

  if (params.find("k")!=params.end()) {
    string k = params.get("k");
    string dk = decode_wapkeys(k);
    for (unsigned int i=0; i<dk.length(); ++i) {
      ses.key_fifo.push(dk[i]);
    }
    Globals::singleton().notify_backend();
  }

  ses.report_any_backend_error();

  usleep(500000);

  ses.report_any_backend_error();

  ostringstream s;
  s << WML_DOCTYPE "\n"
    "<wml>\n"
    "<card title=\"Anyterm\" id=\"term\">\n"
    "<do type=\"accept\" label=\"Send\">\n"
    "<go href=\"anyterm-module\">\n"
    "<postfield name=\"wap\" value=\"\"/>\n"
    "<postfield name=\"s\" value=\"" << ses.id.str() << "\"/>\n"
    "<postfield name=\"k\" value=\"$k\"/>\n"
    "</go>\n"
    "</do>\n"
    "<do type=\"X-refresh\" label=\"Refresh\">\n"
    "<go href=\"anyterm-module\">\n"
    "<postfield name=\"wap\" value=\"\"/>\n"
    "<postfield name=\"s\" value=\"" << ses.id.str() << "\"/>\n"
    "</go>\n"
    "</do>\n"
    "<p>" << wmlify_screen(ses.screen)
    << "</p>\n"
    "<p><input type=\"text\" maxlength=\"127\" name=\"k\" emptyok=\"true\"/></p>\n"
    "</card>\n"
    "</wml>\n";
  
  return wml_resp(s.str());
}


response_t wapterm(const CgiParams& params, string command)
{
  try {
    return wapterm_i(params,command);
  }
  
  catch (Exception& E) {

    ostringstream s;
    s << WML_DOCTYPE "\n"
      "<wml>\n"
      "<card>\n"
      "<p>Exception: ";
    E.report(s);
    s << "</p>\n"
      "</card>\n"
      "</wml>\n";
    
    return wml_resp(s.str());
  }
}


