// src/Exception.hh
// This file is part of libpbe; see http://decimail.org
// (C) 2004 Philip Endecott

// This is version $Name$
//   (if there is no version (e.g. V0-1) mentioned in the previous line,
//    this is probably a snapshot from between "official" releases.)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#ifndef libpbe_Exception_hh
#define libpbe_Exception_hh

#include <errno.h>

#include <iostream>

using namespace std;


class Exception
{
public:
  Exception(void): exit_status(1) {}
  virtual ~Exception() {}
  virtual void report(ostream& s) = 0;
  const int exit_status;
};


class StrException: public Exception
{
private:
  const string msg;
public:
  StrException(string s): msg(s) {}
  void report(ostream& s) { s << msg << endl; }
};


class StdException: public Exception
{
private:
  string msg;
public:
  StdException(exception& e): msg(e.what()) {}
  void report(ostream& s) { s << msg << endl; }
};


class UnknownException: public Exception
{
public:
  void report(ostream& s);
};


class SysException: public Exception
{
private:
  int error_number;
  string doing_what;

public:
  SysException(string dw): error_number(errno), doing_what(dw) {}
  int get_errno(void) { return error_number; }
  void report(ostream& s);
};


#define RETHROW_MISC_EXCEPTIONS \
catch(Exception& E) {           \
  throw;                        \
}                               \
catch(exception& e) {           \
  throw StdException(e);        \
}                               \
catch(const char* s) {          \
  throw StrException(s);        \
}                               \
catch(string s) {               \
  throw StrException(s);        \
}                               \
catch(...) {                    \
  throw UnknownException();     \
}



#endif
